@@ -2,7 +2,7 @@
 # Copyright (C) 2019 FreeIPA Contributors see COPYING for license
 #
 
-from os import environ
+import os
 import sys
 
 from ipahealthcheck.core import constants
@@ -32,7 +32,7 @@ class IPAChecks(RunChecks):
                             help='File to read as input')
         parser.add_argument('--failures-only', dest='failures_only',
                             action='store_true', default=False,
-                            help='Exclude SUCCESS results on output (see'
+                            help='Exclude SUCCESS results on output (see '
                             'man page for more details)')
         parser.add_argument('--all', dest='all',
                             action='store_true', default=False,
@@ -43,8 +43,10 @@ class IPAChecks(RunChecks):
 
 
 def main():
-    environ["KRB5_CLIENT_KTNAME"] = "/etc/krb5.keytab"
-    environ["KRB5CCNAME"] = "MEMORY:"
+    if not os.getegid() == 0:
+        sys.exit("\nYou must be root to run this script.\n")
+    os.environ["KRB5_CLIENT_KTNAME"] = "/etc/krb5.keytab"
+    os.environ["KRB5CCNAME"] = "MEMORY:"
 
     ipachecks = IPAChecks(['ipahealthcheck.registry',
                            'pkihealthcheck.registry'],
