@@ -5,6 +5,7 @@ from __future__ import division
 
 from datetime import datetime, timezone, timedelta
 import itertools
+from inspect import signature
 import logging
 import os
 import socket
@@ -34,6 +35,8 @@ from ipapython.ipaldap import realm_to_serverid
 logger = logging.getLogger()
 DAY = 60 * 60 * 24
 
+expected_requests = []  # to cache the expected certmonger requests
+
 
 def is_ipa_issued_cert(myapi, cert):
     """Thin wrapper around certs.is_ipa_issued to test for LDAP"""
@@ -43,7 +46,7 @@ def is_ipa_issued_cert(myapi, cert):
     return certs.is_ipa_issued_cert(myapi, cert)
 
 
-def get_expected_requests(ca, ds, serverid):
+def get_expected_requests(ca, ds, serverid, conn):
     """Provide the expected certmonger tracking request data
 
        This list is based in part on certificate_renewal_update() in
@@ -53,10 +56,15 @@ def get_expected_requests(ca, ds, serverid):
        The list is filtered depending on whether a CA is running
        and the certificates have been issued by IPA.
 
-      :param ca: the CAInstance
-      :param ds: the DSInstance
-      :param serverid: the DS serverid name
+       :param ca: the CAInstance
+       :param ds: the DSInstance
+       :param serverid: the DS serverid name
     """
+    global expected_requests
+
+    if expected_requests:
+        return expected_requests
+
     template = paths.CERTMONGER_COMMAND_TEMPLATE
 
     if api.Command.ca_is_enabled()['result']:
@@ -79,6 +87,14 @@ def get_expected_requests(ca, ds, serverid):
             dogtag_reqs = itertools.chain(dogtag_reqs,
                                           kra.tracking_reqs.items())
         for nick, profile in dogtag_reqs:
+            if profile in ('caSignedLogCert', 'caOCSPCert',
+                           'caSubsystemCert', 'caCACert',
+                           'caAuditSigningCert', 'caTransportCert',
+                           'caStorageCert'):
+                token = ca.token_name
+            else:
+                token = None
+
             req = {
                 'cert-database': paths.PKI_TOMCAT_ALIAS_DIR,
                 'cert-nickname': nick,
@@ -88,7 +104,10 @@ def get_expected_requests(ca, ds, serverid):
                     (template % 'renew_ca_cert "{}"'.format(nick)),
                 'template-profile': profile,
             }
+            if token and token != 'internal':
+                req['key-token'] = token
             requests.append(req)
+        requests.extend(get_lwca_requests(conn))
     else:
         logger.debug('CA is not configured, skipping CA tracking')
 
@@ -163,9 +182,87 @@ def get_expected_requests(ca, ds, serverid):
                 }
             )
 
+    expected_requests = requests
+
     return requests
 
 
+def get_lwca_requests(conn):
+    """
+    Retrieve the known LWCA certificates.
+    """
+    lwca_requests = []
+    try:
+        entries, truncated = conn.find_entries(
+            base_dn=DN(api.env.container_ca, api.env.basedn),
+            filter="(&(objectclass=ipaca)(!(cn=ipa)))",
+        )
+    except (errors.EmptyResult, errors.NotFound) as e:
+        logger.debug("Search for LWCA entries result: %s", e)
+        return lwca_requests
+
+    template = paths.CERTMONGER_COMMAND_TEMPLATE
+    for entry in entries:
+        try:
+            id = entry.get('ipacaid')[0]
+        except Exception as e:
+            logger.debug("Malformed LDAP entry, unable to get LWCA id %s: %s",
+                         entry.get('dn'), e)
+            continue
+        nickname = f'caSigningCert cert-pki-ca {id}'
+
+        req = {
+            'cert-database': paths.PKI_TOMCAT_ALIAS_DIR,
+            'cert-nickname': nickname,
+            'ca-name': RENEWAL_CA_NAME,
+            'cert-presave-command': template % 'stop_pkicad',
+            'cert-postsave-command':
+                (template % 'renew_ca_cert "{}"'.format(nickname)),
+            'template-profile': 'caCACert',
+        }
+        lwca_requests.append(req)
+
+    return lwca_requests
+
+
+def expected_token(token_name, certmonger_token):
+    """The value is stored in two places, do some sanity checking"""
+    if token_name != certmonger_token:
+        logger.debug(
+            "The IPA token %s doesn't match the certmonger token "
+            "%s.", token_name, certmonger_token
+        )
+        return False
+
+    return True
+
+
+def get_token_password(hsm_enabled, token):
+    if not hsm_enabled:
+        return None
+
+    with open(paths.PKI_TOMCAT_PASSWORD_CONF, "r") as passfile:
+        contents = passfile.readlines()
+
+    for line in contents:
+        data = line.split('=', 1)
+        if data[0] == 'hardware-' + token:
+            return data[1]
+
+    return None
+
+
+def get_token_password_file(hsm_enabled, token):
+    """The CA contains the list of HSM passwords, find ours"""
+    pwdfile = None
+    token_pw = get_token_password(hsm_enabled, token)
+
+    if hsm_enabled and token_pw:
+        pwdfile = ipautil.write_tmp_file(token_pw)
+
+    return pwdfile
+
+
 def get_dogtag_cert_password():
     """Return the NSSDB token password
 
@@ -203,10 +300,10 @@ class IPACertmongerExpirationCheck(IPAPlugin):
             request = certmonger._cm_dbus_object(cm.bus, cm, req,
                                                  certmonger.DBUS_CM_REQUEST_IF,
                                                  certmonger.DBUS_CM_IF, True)
-            id = request.prop_if.Get(certmonger.DBUS_CM_REQUEST_IF,
-                                     'nickname')
-            notafter = request.prop_if.Get(certmonger.DBUS_CM_REQUEST_IF,
-                                           'not-valid-after')
+            id = str(request.prop_if.Get(certmonger.DBUS_CM_REQUEST_IF,
+                                         'nickname'))
+            notafter = int(request.prop_if.Get(certmonger.DBUS_CM_REQUEST_IF,
+                                               'not-valid-after'))
             if notafter == 0:
                 yield Result(self, constants.ERROR,
                              key=id,
@@ -261,11 +358,11 @@ class IPACertfileExpirationCheck(IPAPlugin):
             request = certmonger._cm_dbus_object(cm.bus, cm, req,
                                                  certmonger.DBUS_CM_REQUEST_IF,
                                                  certmonger.DBUS_CM_IF, True)
-            id = request.prop_if.Get(certmonger.DBUS_CM_REQUEST_IF,
-                                     'nickname')
+            id = str(request.prop_if.Get(certmonger.DBUS_CM_REQUEST_IF,
+                                         'nickname'))
 
-            store = request.prop_if.Get(certmonger.DBUS_CM_REQUEST_IF,
-                                        'cert-storage')
+            store = str(request.prop_if.Get(certmonger.DBUS_CM_REQUEST_IF,
+                                            'cert-storage'))
             if store == 'FILE':
                 certfile = str(request.prop_if.Get(
                                certmonger.DBUS_CM_REQUEST_IF, 'cert-file'))
@@ -280,12 +377,26 @@ class IPACertfileExpirationCheck(IPAPlugin):
                                      'file \'{certfile}\': {error}')
                     continue
             elif store == 'NSSDB':
+                token = str(request.prop_if.Get(certmonger.DBUS_CM_REQUEST_IF,
+                                                'key-token'))
                 nickname = str(request.prop_if.Get(
                                certmonger.DBUS_CM_REQUEST_IF, 'key_nickname'))
+                if token and expected_token(self.ca.token_name, token):
+                    nickname = '{}:{}'.format(token, nickname)
                 dbdir = str(request.prop_if.Get(
                             certmonger.DBUS_CM_REQUEST_IF, 'cert_database'))
+
+                pwd_file = get_token_password_file(self.ca.hsm_enabled,
+                                                   token)
+
                 try:
-                    db = certdb.NSSDatabase(dbdir)
+                    if 'pwd_file' in signature(certdb.NSSDatabase).parameters:
+                        db = certdb.NSSDatabase(
+                            dbdir, token=token,
+                            pwd_file=pwd_file.name if pwd_file else None)
+                    else:
+                        # Fall back to older API
+                        db = certdb.NSSDatabase(dbdir)
                 except Exception as e:
                     yield Result(self, constants.ERROR,
                                  key=id,
@@ -315,8 +426,9 @@ class IPACertfileExpirationCheck(IPAPlugin):
                                  'storage type: {store}')
                 continue
 
-            now = datetime.utcnow()
-            notafter = cert.not_valid_after
+            now = datetime.now(tz=timezone.utc)
+            # Older versions of IPA provide naive timestamps
+            notafter = cert.not_valid_after.replace(tzinfo=timezone.utc)
 
             if now > notafter:
                 yield Result(self, constants.ERROR,
@@ -362,7 +474,8 @@ class IPACertTracking(IPAPlugin):
 
     @duration
     def check(self):
-        requests = get_expected_requests(self.ca, self.ds, self.serverid)
+        requests = get_expected_requests(self.ca, self.ds, self.serverid,
+                                         self.conn)
         cm = certmonger._certmonger()
 
         ids = []
@@ -371,9 +484,9 @@ class IPACertTracking(IPAPlugin):
             request = certmonger._cm_dbus_object(cm.bus, cm, req,
                                                  certmonger.DBUS_CM_REQUEST_IF,
                                                  certmonger.DBUS_CM_IF, True)
-            id = request.prop_if.Get(certmonger.DBUS_CM_REQUEST_IF,
-                                     'nickname')
-            ids.append(str(id))
+            id = str(request.prop_if.Get(certmonger.DBUS_CM_REQUEST_IF,
+                                         'nickname'))
+            ids.append(id)
 
         for request in requests:
             request_id = certmonger.get_request_id(request)
@@ -431,7 +544,8 @@ class IPACertDNSSAN(IPAPlugin):
     @duration
     def check(self):
         fqdn = socket.getfqdn()
-        requests = get_expected_requests(self.ca, self.ds, self.serverid)
+        requests = get_expected_requests(self.ca, self.ds, self.serverid,
+                                         self.conn)
 
         for request in requests:
             request_id = certmonger.get_request_id(request)
@@ -525,31 +639,50 @@ class IPACertDNSSAN(IPAPlugin):
 @registry
 class IPACertNSSTrust(IPAPlugin):
     """Compare the NSS trust for the CA certs to a known good value"""
+
     @duration
     def check(self):
+        if not self.ca.is_configured():
+            logger.debug('CA is not configured, skipping NSS trust check')
+            return
+
+        if self.ca.hsm_enabled:
+            token = self.ca.token_name + ":"
+        else:
+            token = ""
+
         expected_trust = {
-            'ocspSigningCert cert-pki-ca': 'u,u,u',
-            'subsystemCert cert-pki-ca': 'u,u,u',
-            'auditSigningCert cert-pki-ca': 'u,u,Pu',
+            f'{token}ocspSigningCert cert-pki-ca': 'u,u,u',
+            f'{token}subsystemCert cert-pki-ca': 'u,u,u',
+            f'{token}auditSigningCert cert-pki-ca': 'u,u,Pu',
             'Server-Cert cert-pki-ca': 'u,u,u',
         }
         kra = krainstance.KRAInstance(api.env.realm)
         if kra.is_installed():
             kra_trust = {
-                'transportCert cert-pki-kra': 'u,u,u',
-                'storageCert cert-pki-kra': 'u,u,u',
-                'auditSigningCert cert-pki-kra': 'u,u,Pu',
+                f'{token}transportCert cert-pki-kra': 'u,u,u',
+                f'{token}storageCert cert-pki-kra': 'u,u,u',
+                f'{token}auditSigningCert cert-pki-kra': 'u,u,Pu',
             }
             expected_trust.update(kra_trust)
 
-        if not self.ca.is_configured():
-            logger.debug('CA is not configured, skipping NSS trust check')
-            return
+        db = certdb.NSSDatabase(paths.PKI_TOMCAT_ALIAS_DIR)
+        certlist = db.list_certs()
+        if token:
+            token = token[:-1]  # Strip off trailing colon
 
-        db = certs.CertDB(api.env.realm, paths.PKI_TOMCAT_ALIAS_DIR)
-        for nickname, _trust_flags in db.list_certs():
+            pwd_file = get_token_password_file(self.ca.hsm_enabled,
+                                               token)
+
+            db = certdb.NSSDatabase(
+                paths.PKI_TOMCAT_ALIAS_DIR, token=token,
+                pwd_file=pwd_file.name if pwd_file else None)
+
+            certlist += db.list_certs()
+
+        for nickname, _trust_flags in certlist:
             flags = certdb.unparse_trust_flags(_trust_flags)
-            if nickname.startswith('caSigningCert cert-pki-ca'):
+            if nickname == f'{token}caSigningCert cert-pki-ca':
                 expected = 'CTu,Cu,Cu'
             else:
                 try:
@@ -706,7 +839,7 @@ class IPADogtagCertsMatchCheck(IPAPlugin):
     @duration
     def check(self):
         if not self.ca.is_configured():
-            logger.debug('CA is not configured, skipping connectivity check')
+            logger.debug('CA is not configured, skipping match check')
             return
 
         def match_ldap_nss_cert(plugin, ldap, db, cert_dn, attr, cert_nick):
@@ -761,9 +894,21 @@ class IPADogtagCertsMatchCheck(IPAPlugin):
                                       '{dbdir} does not match entry in LDAP'))
             return all_ok
 
-        db = certs.CertDB(api.env.realm, paths.PKI_TOMCAT_ALIAS_DIR)
+        if self.ca.hsm_enabled:
+            token = self.ca.token_name + ':'
+        else:
+            token = ''
+        pwd_file = get_token_password_file(self.ca.hsm_enabled,
+                                           self.ca.token_name)
+        if 'pwd_file' in signature(certs.CertDB).parameters:
+            # pylint: disable=unexpected-keyword-arg
+            db = certs.CertDB(api.env.realm, paths.PKI_TOMCAT_ALIAS_DIR,
+                              pwd_file=pwd_file.name if pwd_file else None)
+        else:
+            # Fall back to older API
+            db = certs.CertDB(api.env.realm, paths.PKI_TOMCAT_ALIAS_DIR)
         dn = DN('uid=pkidbuser,ou=people,o=ipaca')
-        subsystem_nick = 'subsystemCert cert-pki-ca'
+        subsystem_nick = f'{token}subsystemCert cert-pki-ca'
         subsystem_ok = yield from match_ldap_nss_cert(self, self.conn,
                                                       db, dn,
                                                       'userCertificate',
@@ -771,7 +916,7 @@ class IPADogtagCertsMatchCheck(IPAPlugin):
         dn = DN('cn=%s IPA CA' % api.env.realm,
                 'cn=certificates,cn=ipa,cn=etc',
                 api.env.basedn)
-        casigning_nick = 'caSigningCert cert-pki-ca'
+        casigning_nick = f'{token}caSigningCert cert-pki-ca'
         casigning_ok = yield from match_ldap_nss_cert(self, self.conn,
                                                       db, dn, 'CACertificate',
                                                       casigning_nick)
@@ -779,11 +924,11 @@ class IPADogtagCertsMatchCheck(IPAPlugin):
         config = api.Command.config_show()
         subject_base = config['result']['ipacertificatesubjectbase'][0]
         expected_nicks_subjects = {
-            'ocspSigningCert cert-pki-ca':
+            f'{token}ocspSigningCert cert-pki-ca':
                 f'CN=OCSP Subsystem,{subject_base}',
-            'subsystemCert cert-pki-ca':
+            f'{token}subsystemCert cert-pki-ca':
                 f'CN=CA Subsystem,{subject_base}',
-            'auditSigningCert cert-pki-ca':
+            f'{token}auditSigningCert cert-pki-ca':
                 f'CN=CA Audit,{subject_base}',
             'Server-Cert cert-pki-ca':
                 f'CN={api.env.host},{subject_base}',
@@ -792,11 +937,11 @@ class IPADogtagCertsMatchCheck(IPAPlugin):
         kra = krainstance.KRAInstance(api.env.realm)
         if kra.is_installed():
             kra_expected_nicks_subjects = {
-                'transportCert cert-pki-kra':
+                f'{token}transportCert cert-pki-kra':
                     f'CN=KRA Transport Certificate,{subject_base}',
-                'storageCert cert-pki-kra':
+                f'{token}storageCert cert-pki-kra':
                     f'CN=KRA Storage Certificate,{subject_base}',
-                'auditSigningCert cert-pki-kra':
+                f'{token}auditSigningCert cert-pki-kra':
                     f'CN=KRA Audit,{subject_base}',
             }
             expected_nicks_subjects.update(kra_expected_nicks_subjects)
@@ -891,7 +1036,7 @@ class IPANSSChainValidation(IPAPlugin):
                         key=key,
                         dbdir=dbdir,
                         nickname=nickname,
-                        reason=response.output_error,
+                        reason=str(e),
                         msg='Validation of {nickname} in {dbdir} failed: '
                             '{reason}')
                 else:
@@ -1021,7 +1166,16 @@ def check_agent(plugin, base_dn, agent_type):
             return
         ra_desc = raw_desc[0]
         ra_certs = entry.get('usercertificate')
-        if ra_desc != description:
+        (_version, exp_serial, exp_issuer, exp_subject) = \
+            ra_desc.split(';')
+        matched = all(
+            [
+                str(serial_number) == exp_serial,
+                DN(issuer) == DN(exp_issuer),
+                DN(subject) == DN(exp_subject),
+            ]
+        )
+        if not matched:
             yield Result(plugin, constants.ERROR,
                          key='description_mismatch',
                          expected=description,
@@ -1121,7 +1275,10 @@ class IPACertRevocation(IPAPlugin):
         if not self.ca.is_configured():
             logger.debug('CA is not configured, skipping revocation check')
             return
-        requests = get_expected_requests(self.ca, self.ds, self.serverid)
+        requests = get_expected_requests(self.ca, self.ds, self.serverid,
+                                         self.conn)
+        pwd_file = get_token_password_file(self.ca.hsm_enabled,
+                                           self.ca.token_name)
         for request in requests:
             id = certmonger.get_request_id(request)
             if request.get('cert-file') is not None:
@@ -1138,9 +1295,19 @@ class IPACertRevocation(IPAPlugin):
                     continue
             elif request.get('cert-database') is not None:
                 nickname = request.get('cert-nickname')
+                token = request.get('key-token')
+                if token == 'internal':
+                    token = None
                 dbdir = request.get('cert-database')
                 try:
-                    db = certdb.NSSDatabase(dbdir)
+                    if 'pwd_file' in signature(certdb.NSSDatabase).parameters:
+                        db = certdb.NSSDatabase(
+                            dbdir, token=token,
+                            pwd_file=pwd_file.name if pwd_file else None
+                        )
+                    else:
+                        # Fall back to older API that doesn't support tokens
+                        db = certdb.NSSDatabase(dbdir)
                 except Exception as e:
                     yield Result(self, constants.ERROR,
                                  key=id,
@@ -1149,6 +1316,8 @@ class IPACertRevocation(IPAPlugin):
                                  msg='Unable to open NSS database {dbdir}: '
                                      '{error}')
                     continue
+                if token:
+                    nickname = f'{token}:{nickname}'
                 try:
                     cert = db.get_cert(nickname)
                 except Exception as e:
@@ -1179,8 +1348,7 @@ class IPACertRevocation(IPAPlugin):
 
             # Now we have the cert either way, check the recovation
             try:
-                result = api.Command.cert_show(cert.serial_number,
-                                               all=True)
+                result = api.Command.cert_show(cert.serial_number)
             except Exception as e:
                 yield Result(self, constants.ERROR,
                              key=id,
@@ -1298,3 +1466,23 @@ class IPACAChainExpirationCheck(IPAPlugin):
                              path=paths.IPA_CA_CRT,
                              key=subject,
                              days=(dt - now).days)
+
+
+@registry
+class CertmongerStuckCheck(IPAPlugin):
+    """Check for certonger requests in the stuck state
+    """
+
+    @duration
+    def check(self):
+        requests = certmonger._get_requests({'stuck': True})
+        for request in requests:
+            id = str(request.prop_if.Get(
+                     certmonger.DBUS_CM_REQUEST_IF, 'nickname'))
+            yield Result(self, constants.WARNING,
+                         key=id,
+                         msg='certmonger request {key} is in the '
+                         'stuck state')
+
+        if len(requests) == 0:
+            yield Result(self, constants.SUCCESS, key='no_stuck')
