@@ -7,7 +7,7 @@ from base import BaseTest
 
 from ipahealthcheck.core import constants, config
 from ipahealthcheck.ipa.plugin import registry
-from ipahealthcheck.ipa.certs import IPACertTracking
+from ipahealthcheck.ipa.certs import IPACertTracking, CertmongerStuckCheck
 from unittest.mock import Mock
 from mock_certmonger import create_mock_dbus, _certmonger
 from mock_certmonger import get_expected_requests, set_requests
@@ -84,3 +84,47 @@ class TestTracking(BaseTest):
         assert result.source == 'ipahealthcheck.ipa.certs'
         assert result.check == 'IPACertTracking'
         assert result.kw.get('key') == '7777'
+
+
+class TestStuck(BaseTest):
+    patches = {
+        'ipahealthcheck.ipa.certs.get_expected_requests':
+        Mock(return_value=get_expected_requests()),
+        'ipalib.install.certmonger._cm_dbus_object':
+        Mock(side_effect=create_mock_dbus),
+        'ipalib.install.certmonger._certmonger':
+        Mock(return_value=_certmonger())
+    }
+
+    def test_none_stuck(self):
+        set_requests()
+
+        framework = object()
+        registry.initialize(framework, config.Config)
+        f = CertmongerStuckCheck(registry)
+
+        self.results = capture_results(f)
+
+        assert len(self.results) == 1
+        result = self.results.results[0]
+        assert result.result == constants.SUCCESS
+
+    def test_one_stuck(self):
+        stuck = {
+            'nickname': '7777',
+            'cert-file': '/tmp/test.crt',
+            'key-file': '/tmp/test.key',
+            'ca-name': 'IPA',
+            'stuck': True,
+        }
+        set_requests(add=stuck)
+
+        framework = object()
+        registry.initialize(framework, config.Config)
+        f = CertmongerStuckCheck(registry)
+
+        self.results = capture_results(f)
+        assert len(self.results) == 1
+        result = self.results.results[0]
+        assert result.result == constants.WARNING
+        assert result.kw.get('key') == '7777'
